package org.gecko.runtime.bnd;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.Collections;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import aQute.bnd.build.Run;
import aQute.bnd.build.Workspace;
import aQute.bnd.osgi.JarResource;
import aQute.bnd.osgi.Resource;
import aQute.lib.io.IO;

public class GeckoBndRunTest {
	File						base					= new File("").getAbsoluteFile();
	private File generatedDir;
	private File result;

	@Before
	public void before() {
		generatedDir = new File(base, "generated/");
	}
	
	@After
	public void after() {
		if(result != null) {
			result.delete();
		}
	}
	
	/**
	 * Testing the embedded launcher is quite tricky. This test uses a prefabricated
	 * packaged jar. Notice that you need to reexport that jar for every change in
	 * the launcher since it embeds the launcher. This jar is run twice to see if
	 * the second run will not reinstall the bundles.
	 */

	@Test
	public void testExport() throws Exception {
		
		
		Workspace ws = Workspace.getWorkspace(base.getParentFile());
//		
		Run run = Run.createRun(ws, IO.getFile(base, "test.bndrun"));

		Entry<String, Resource> export = run.export("gecko.export", Collections.emptyMap());
		assertNotNull(export);
		assertNotNull(export.getValue());
		
		assertTrue(export.getValue() instanceof JarResource);
		
		JarResource exportResult = (JarResource) export.getValue();
		result = new File(generatedDir, export.getKey().replace(".jar", ".zip"));
		exportResult.getJar().write(result);
		
		assertTrue(result.exists());
		
	}
	
	public static boolean deleteDir(File dir) 
	{ 
	  if (dir.isDirectory()) 
	  { 
	    String[] children = dir.list(); 
	    for (int i=0; i<children.length; i++)
	      deleteDir(new File(dir, children[i])); 
	  }  
	  // The directory is now empty or this is a file so delete it 
	  return dir.delete(); 
	} 
}
